/*
** Module   : Driver.java
** Abstract : thread that reads tests from a job queue and executes them
**
** Copyright (c) 2009, Golden Code Development Corporation.
**
** -#- -I- --Date-- -----------------------Description------------------------
** 001 GES 20090122 Thread that reads tests from a job queue and executes them.
*/

/*
** This program is free software: you can redistribute it and/or modify
** it under the terms of the GNU Affero General Public License as
** published by the Free Software Foundation, either version 3 of the
** License, or (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU Affero General Public License for more details.
**
** You should have received a copy of the GNU Affero General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package com.goldencode.harness;

import java.io.*;
import java.util.concurrent.*;

import com.goldencode.harness.transport.*;
import com.goldencode.harness.terminal.*;
import com.goldencode.harness.test.*;


/**
 * Thread that reads tests from a job queue and executes them.
 * <p>
 * Implements the core logic of a session worker thread. This provides a
 * single thread that reads from the job queue (which provides tests to
 * execute) and executes each test in order against a specific target back
 * end system.
 */
public class Driver
extends Thread
{
   /** Low-level communications transport. */ 
   Connection session = null;
   
   /** The terminal emulator session to the target back-end. */
   Terminal term = null;
   
   /** Job queue containing the tests to run. */
   JobQueue<Test> queue = null;
   
   /** Object to notify when complete. */
   CountDownLatch latch = null;
   
   /**
    * Construct an instance of a worker thread and establish the connection
    * to the back-end. This includes the creation of the terminal emulator
    * instance which is used for test interaction.
    *
    * @param    cfg
    *           Communications session and terminal emulation configuration       *
    *           for properly contacting the target system.
    * @param    queue
    *           Job queue from which tests to be executed are obtained.
    * @param    latch
    *           Object to notify when complete.
    * @param    rc
    *           Object in which to store the details of non-testing failures.
    */
   public Driver(SessionSettings     cfg,
                 JobQueue<Test>      queue,
                 CountDownLatch      latch,
                 FailureNotification rc)
   {
      this.queue = queue;
      this.latch = latch;
      
      // connect the session
      session = TransportManager.connect(cfg, rc);
      
      if (session == null)
      {
         // failure details are already in the rc
         return;
      }
      
      OutputStream os = session.getOutputStream(rc);
      InputStream  is = session.getInputStream(rc);
      
      if (os == null || is == null)
      {
         // failure details are already in the rc
         return;
      }
      
      term = new Terminal(new DataSource(is, os), cfg);
   }
   
   /**
    * Execute tests obtained from the job queue.
    */
   public void run()
   {
      try
      {
         if (session == null)
            return;
         
         loop:
         while (true)
         {
            Test test = queue.dequeue();
            
            if (test == null)
               break;
            
            test.execute(term);
            
            Status status = test.getStatus();
            
            if (status == Status.FAILED)
            {
               FailureMode fail = test.getFailureMode();
               
               switch (fail)
               {
                  case IGNORE:
                     continue loop;
                     
                  case ABORT_THREAD:
                     break loop;
                     
                  case BACKOUT:
                     Test backout = test.getBackout();
                     
                     if (backout != null)
                        backout.execute(term);
                     
                     break;
               }
            }
         }
      }
      
      finally
      {
         if (session != null)
            session.disconnect();
         
         latch.countDown();
      }
   }
}
